# -*-perl-*-

use strict;
use Win32::OLE qw(in);
use File::Spec;
use LWP::Simple;
use Image::Size;
use Encode;
use constant {
    ITTrackKindFile => 1,
    ITPlaylistKindLibrary => 1,
    ITPlaylistKindUser => 2,
};

our $C;
our $itunes_playlist;

$C->{iTunes}{SelectPlayList}	||= \&itunes_get_playlist;
$C->{iTunes}{GetAttlibs}	||= \&itunes_get_attribs;
$C->{iTunes}{AfterAdded}	||= \&noop;

our $iTunes;
my $source;

sub add_to_itunes {
    eval {
	my ($src_s, $dst_s, $meta, $taginfo) = @_;

	unless ($dst_s =~ /\.(?:mp[34]|m4a|aac)$/) {
	    logger("iTunes: ignored non-mp3/mp4 file: $dst_s");
	    return;
	}

	unless ($iTunes) {
	    itunes_init();
	}

	my $pl = $C->{iTunes}{SelectPlayList}->($src_s, $dst_s, $meta, $taginfo);
	defined($pl) or $pl = $iTunes->LibraryPlaylist;

	my @tracks = itunes_add_file_unique($pl, $dst_s);
	if (@tracks == 1) {
	    my $t = $tracks[0];

	    # set tags
	    my $attrib = $C->{iTunes}{GetAttlibs}->($src_s, $dst_s, $meta, $taginfo);
	    for my $k (keys %$attrib) {
		eval {
		    my $v = sjis($attrib->{$k});
		    $t->{$k} = $v;
		    if (Win32::OLE->LastError) {
			logger("itunes: error setting attrib $k -> $v: " .
			      sprintf("0x%08x", Win32::OLE->LastError));
			if (Win32::OLE->LastError == 0x80020009) {
			    sleep(1);
			    $t->{$k} = $v;
			}
		    }
		};
		$@ and error($@);
	    }
	    sleep(1);

	    # set thumbnail
	    if ($C->{iTunes}{UseNicoThumbNail}) {
		itunes_add_thumbnail_from_nico($t, $src_s, $dst_s, $meta, $taginfo);
	    }

	    $C->{iTunes}{AfterAdded}->($t, $src_s, $dst_s, $meta, $taginfo);
	}
    };
    $@ and error($@);
}

sub itunes_init {
    $iTunes = Win32::OLE->new('iTunes.Application');
    $source = $iTunes->LibrarySource;
}

sub itunes_get_playlist {
    my $pl = $itunes_playlist;
    if ($itunes_playlist eq '') {
	return undef;
    }

    my $p = utf8($pl);
    my $pp = itunes_find_playlists(qr/^\Q$p\E$/);
    my $ret = undef;
    if (scalar(keys(%$pp)) == 1) {
	$ret = (values %$pp)[0];
    } elsif (scalar(keys(%$pp)) == 0) {
	logger("Playlist $pl not found.  Add to library");
	$ret = undef;
    } else {
	logger("Playlist $pl ambiguous.  Add to library");
	$ret = undef;
    }
    $ret;
}

my %iTunesAttribs = (
		    artist => 'Artist',
		    comment => 'Comment',
		    album => 'Album',
		    title => 'Name',
		    genre => 'Genre',
		    year => 'Year',
		    track => 'TrackNumber',
);

sub itunes_get_attribs {
    my ($src_s, $dst_s, $meta, $taginfo) = @_;
    my $ret = {};
 
    for my $k (keys %iTunesAttribs) {
	my $v = $meta->{$k} || $meta->{uc $k} ||
	    $taginfo->{$k} || $taginfo->{uc $k};
	if (defined($v) && $v ne '') {
	    $ret->{$iTunesAttribs{$k}} = $v;
	}
    }
    $ret;
}

sub itunes_add_file_unique {
    my ($playlist, $file) = @_;

    if ($playlist->Kind != ITPlaylistKindLibrary) {
	for my $t (in($playlist->Tracks)) {
	    if ($t->Kind == ITTrackKindFile &&
		$t->Location eq $file) {
		logger("iTunes: already on the list: ", $t->Name, " -> ", $playlist->Name);
		return ($t);
	    }
	}
    }

    my $stat = $playlist->AddFile($file);

    while ($stat->InProgress) {
	last if Win32::OLE->LastError();
	sleep(1);
    }
    return undef if Win32::OLE->LastError();

    my @tracks = in($stat->Tracks);
    if (@tracks > 0) {
	logger("iTunes: added: ", $tracks[0]->Name, " -> ", $playlist->Name);
    }
    @tracks;
}

sub itunes_find_playlists {
    my $regex = shift;
    my $ret = {};
    for my $playlist (in($source->Playlists)) {
	my $name = utf8($playlist->Name);
	if ($name =~ $regex) {
	    $ret->{$name} = $playlist;
	}
    }
    $ret;
}

sub itunes_add_thumbnail_from_nico {
    my ($t, $src_s, $dst_s, $meta, $taginfo) = @_;
    $t->Artwork->Count and return;

    my $nicoid = $meta->{nicoid} || $taginfo->{nicoid};
    unless (defined($nicoid)) {
	($nicoid) = (utf8($dst_s) =~ /\b($C->{NicoPrefix}\d{4,})/o);
    }

    my ($numid) = $nicoid =~ /(\d+)/;
    return unless defined $numid;

    my $tmpfile = tmpfile("jpg");
    unlink $tmpfile;
    getstore("http://tn-skr.smilevideo.jp/smile?i=$numid" => $tmpfile);
    my ($x, $y) = imgsize($tmpfile);

    if ($x > 1 && $y > 1) {
	$t->AddArtworkFromFile($tmpfile);
	sleep(1);
    }
    unlink $tmpfile;
}

1;
