# $Id: parse_vocaloid.pl,v 1.11 2008/04/09 22:11:38 maeda Exp $

# Copyright 2008 by Kaoru Maeda <kaoru.maeda@gmail.com>
#
# This program is free software; you may redistribute it and/or modify it
# under the same terms as Perl itself.

use strict;
use utf8;
use Encode;

our $C;

sub parse_filename {
    my $src_u = decode('sjis', $_[0]);

    $src_u =~ s/^.*\\//;
    $src_u =~ s[^.*/][];

    my ($id) = $src_u =~ m[^($C->{NicoPrefix}\d+)]o;
    my $title = $src_u;
    $title =~ s/^$C->{NicoPrefix}\d+_//o;
    $title =~ s/\.$C->{InputType}$//o;

    my @info;
    if ($src_u =~ /【.*?】/) {
	@info = $src_u =~ /【(.*?)】/g;
	$title =~ s/【.*?】//g;
    } elsif ($src_u =~ /『.*?』/) {
	@info = $src_u =~ /『(.*?)』/g;
	$title =~ s/『.*?』//g;
	if (@info == 1) {
	    # swap info and title
	    ($info[0], $title) = ($title, $info[0]);
	}
    }
    if ($title =~ /\((.*?)\)/) {
	push(@info, $title =~ /\((.*?)\)/g);
	$title =~ s/\(.*?\)//g;
    } elsif ($title =~ /\[(.*?)\]/) {
	push(@info, $title =~ /\[(.*?)\]/g);
	$title =~ s/\[.*?\]//g;
    } elsif ($title =~ /（(.*?)）/) {
	push(@info, $title =~ /（(.*?)）/g);
	$title =~ s/（(.*?)）//g;
    } elsif ($title =~ /＜(.*?)＞/) {
	push(@info, $title =~ /＜(.*?)＞/g);
	$title =~ s/＜(.*?)＞//g;
    }

    my @newinfo;
    my @artists;
    for (@info, $title) {
	if (s/オリジナル曲?//) {
	    push(@newinfo, 'オリジナル');
	}
	if (s/(VOCALOID\d?|(?:full|short|long|フル|修正|高音質|高画質)(?:[\s.]*(?:ver\.|version|バージョン|版|edition|edit)?)|mikumikudance)//i) {
	    push(@newinfo, $1);
	}
	while (s/(?:・|･|＋|,\s*|，\s*|＆|with\s*|and\s*)?(初音ミク|鏡音リン(?:・レン)?|鏡音レン|meiko|Ｍｅｉｋｏ|ＭＥＩＫＯ|kaito|Ｋａｉｔｏ|ＫＡＩＴＯ|Sweet *Ann|Leon|(?<=・)(?:リン|レン|ミク)|(?:リン|レン|ミク)(?=・))//i) {
	    my $a = $1;
	    $a =~ s/meiko|Ｍｅｉｋｏ|ＭＥＩＫＯ/MEIKO/i;
	    $a =~ s/kaito|Ｋａｉｔｏ|ＫＡＩＴＯ/KAITO/i;
	    push(@artists, $a);
	}
    }
    push(@info, @newinfo);

    for (@info) {
	s/　/ /g; # 全角空白
	s/が?\s*を?\s*(?:歌って(?:くれ|み)[ぁ-ん]+)//;
	s/[,，×+＋・、]\s*//g;
	s/\s+-\s+//g;
	s/^\s+//;
	s/\s+$//;
	s/;\"/-/g;
    }
    @info = grep {$_ ne ''} @info;

    for ($title) {
	s/　/ /g; # 全角空白
	s/^\s*東方アレンジ(?:vocal)?//i;
	s/が\s*を\s*(?:歌って(?:くれ|み)[ぁ-ん]+)//;
	s/^\s+//;
	s/\s+$//;
	s/;\"/-/g;
    }

    my $comments = join(", ", @info);
    my $artists = join("・", @artists);

    my $meta = {
	nicoid => $id,
	album => $id,
	title => $title,
	artist => $artists,
	comment => $comments,
    };
}

1;
