# -*-perl-*-

# $Id: userdef.pl,v 1.8 2008/04/11 19:07:22 maeda Exp $

# Example userdef.pl
# Encoding: このファイルはutf8

use strict;
use utf8;
use Encode;
use Data::Dumper;

our $C;
our $format;
our $itunes_playlist;

$C->{Format}{"radio"}{Ext} = 'mp3';
$C->{Format}{"radio"}{FFMpeg} = "-f mp3 -ac 1 -ar 22050 ";
$C->{Format}{"radio"}{AimedAudioBP} = 48;
$C->{Format}{"radio"}{ABPLadder} = [64, 48, 32];
$C->{Format}{"radio"}{Description} = sjis("mp3 音声: モノ/低ビットレート ネトラジ/PodCastトーク向け");
$C->{Format}{"radio"}{NoVideo} = 1;

$C->{Format}{"nokia"}{Ext} = '3gp';
$C->{Format}{"nokia"}{FFMpeg} = "-f 3gp -vcodec h263 -acodec libfaac -ac 1 -ar 8000 ";
$C->{Format}{"nokia"}{AimedAudioBP} = 32;
$C->{Format}{"nokia"}{AimedVideoBP} = 96;
$C->{Format}{"nokia"}{VBPLadder} = [128, 96, 64];
$C->{Format}{"nokia"}{ABPLadder} = [32];
$C->{Format}{"nokia"}{AimedVideoWidth} = 176;
$C->{Format}{"nokia"}{VWLadder} = [352,176,128];
$C->{Format}{"nokia"}{Description} = sjis("3gp 動画: Nokia 6630 向け");
$C->{Format}{"nokia"}{NoVideo} = 0;
$C->{Format}{"nokia"}{ForceAspect} = [qw(352x288 176x144 128x96)];
$C->{Format}{"nokia"}{FrameRate} = 15;

$C->{iTunes}{SelectPlayList}	= \&my_get_playlist;
$C->{iTunes}{GetAttlibs}	= \&my_get_attribs;

$C->{TagCleaner} = \&gettag;
sub gettag {
    my ($src_s, $dst_s, $meta) = @_;
    my $srcfile_u = $meta->{srcfile};
    my $src_u = utf8($src_s);
    my $dst_u = utf8($dst_s);
    my $srcfile_s = sjis($srcfile_u);

    $meta->{genre} gt '' or $meta->{genre} = "Other";

    if ($src_u =~ /RADIO ARIMAX/||
	$srcfile_u =~ /RADIO ARIMAX/||
	$dst_u =~ /RADIO ARIMAX/) {
	my $file = -f $srcfile_s ? $srcfile_s : $src_u;
	use MP3::Tag;
	my $mp3 = MP3::Tag->new($file);
	$mp3->get_tags;
	my $v2 = $mp3->{ID3v2};
	my $t = $v2 ? $v2->title : utf8($file);
	if ($t =~ /(mikoラジ.*?回)/) {
	    $meta->{title} = $1;
	} elsif ($t =~ /RADIO ARIMAX 第\d+回 (.*?)\（/) {
	    $meta->{title} = $1;
	}
	if ($t =~ m[20\d\d/(\d\d/\d\d)]) {
	    $meta->{title} .= " $1";
	}
    }
}

my $playlists;

sub my_get_playlist {
    my ($src_s, $dst_s, $meta, $taginfo) = @_;

    my $pl = undef;

    if ($itunes_playlist ne '') {
	$pl = itunes_get_playlist();
	defined($pl) and return $pl;
    }

    unless ($playlists) {
	$playlists = itunes_find_playlists(qr/^(?:未評価-|Radio)/);
    }

    my $dst_u = utf8($dst_s);
    if ($format eq 'radio') {
	$pl = $playlists->{"Radio"};
    } elsif ($dst_u =~ /VOCALOID/) {
	$pl = $playlists->{"未評価-VOCALOID"};
    } elsif ($dst_u =~ /東方/) {
	$pl = $playlists->{"未評価-東方"};
    }
    $pl ||= $playlists->{"未評価-その他"};
    $pl;
}

sub my_get_attribs {
    my ($src_s, $dst_s, $meta, $taginfo) = @_;
    my $ret = itunes_get_attribs(@_);
    $ret->{Genre} = '';
    $ret->{ExcludeFromShuffle} = 1;
    $ret->{VolumeAdjustment} = -25;

    if ($format eq 'radio') {
	$ret->{RememberBookmark} = 1;
    }
    $ret;
}

1;
